﻿
local context
local frame

local scanWords = { "stun", "incap", "movement", "root", "silence", "fear", "polymorph", "snared", "confused" }

local function LoadEnd()
	-- save captured cc between sessions
	CrowdedDB = CrowdedDB or {}
	CrowdedDB.drag = CrowdedDB.drag or false
	CrowdedDB.size = CrowdedDB.size or 64
	CrowdedDB.cc = CrowdedDB.cc or {}
end

local function cf(type, p, w, h, x, y)
	local f = UI.CreateFrame(type, "Crowded", context)
	f:SetPoint("CENTER", p, "CENTER", x or 0, y or 0)
	f:SetWidth(w)
	f:SetHeight(h)
	return f
end

local function CreateFrames()
	context = UI.CreateContext("Crowded")
	frame = cf("Frame", UIParent, CrowdedDB.size, CrowdedDB.size, CrowdedDB.x, CrowdedDB.y)

	frame.icon = cf("Texture", frame, CrowdedDB.size, CrowdedDB.size)
	frame.icon:SetLayer(3)
	
	local bordersize = CrowdedDB.size + 4
	frame.border = cf("Texture", frame.icon, bordersize, bordersize)
	frame.border:SetBackgroundColor(0, 0, 0)
	frame.border:SetLayer(2)
	frame.border:SetParent(frame.icon)
	
	frame.drag = cf("Texture", frame, bordersize, bordersize)
	frame.drag:SetBackgroundColor(0, 128, 0)
	frame.drag:SetLayer(4)
	frame.drag:SetAlpha(0.65)

	frame.drag:SetVisible(CrowdedDB.drag)

	function frame.drag.Event:LeftDown()
		frame.mouseData = Inspect.Mouse()
	end

	function frame.drag.Event:MouseMove()
		if frame.mouseData then
			local md = Inspect.Mouse()
			CrowdedDB.x = (md.x - frame.mouseData.x)
			CrowdedDB.y = (md.y - frame.mouseData.y)
			frame:SetPoint("CENTER", UIParent, "CENTER", CrowdedDB.x, CrowdedDB.y)
		end
	end
	function frame.drag.Event:LeftUp()
		frame.mouseData = nil
	end
	frame.drag.Event.LeftUpoutside = frame.drag.Event.LeftUp
end

local function SetIcon(buff)
	-- check to see if it's new
	if buff then
		if buff.ability ~= frame.ability then
			--print("cc!", buff.name)
			frame.icon:SetVisible(true)
			frame.icon:SetTexture("Rift", buff.icon)
			frame.ability = buff.ability
		end
	else
		-- clear it
		frame.icon:SetVisible(false)
		frame.ability = nil
	end
end

local function Update(unit)	
	unitDetails = Inspect.Unit.Detail(unit)
	playerDetails = Inspect.Unit.Detail('player')
	if playerDetails == nil then
		--print("no target")
		return
	end

	if unitDetails.name ~= playerDetails.name then
		--print("wrong unit", unitDetails.name)
		return
	end

	--print("update", unitDetails.name)

	if not context then
		CreateFrames()
	end

	buffs = Inspect.Buff.List(unit)
	buffDetails = Inspect.Buff.Detail(unit, buffs)
	for id, buff in pairs(buffDetails) do
		--abilityDetail = Inspect.Ability.Detail(buff.ability)
		if buff.debuff and buff.ability ~= nil then
			if CrowdedDB.cc[buff.ability] then
				SetIcon(buff)
				return
			elseif (buff.description) then
				-- scan
				for i = 1, #scanWords do
					-- check for scan words
					if string.find(string.lower(buff.description), scanWords[i]) ~= nil then
						print("new cc!", buff.name, buff.description)
						CrowdedDB.cc[buff.ability] = true
						SetIcon(buff)
						return
					end
				end
			end
		end
	end

	-- no cc
	SetIcon(nil)
end

local function Add(unit, buffs)
	Update(unit)
end

local function Remove(unit, buffs)
	Update(unit)
end

local function toggledrag()
	CrowdedDB.drag = not CrowdedDB.drag
	frame.drag:SetVisible(CrowdedDB.drag)
end

table.insert(Event.Addon.Load.End, {LoadEnd, "Crowded", "LoadEnd"})
table.insert(Event.Buff.Add, {Add, "Crowded", "buffAdded"})
table.insert(Event.Buff.Remove, {Remove, "Crowded", "buffRemoved"})

table.insert(Command.Slash.Register("crowded"), {toggledrag, "Crowded", "crowded"})
